@@ -8,16 +8,14 @@ from os.path import (join as pjoin, dirname, basename, exists, abspath,
                      relpath, realpath)
 import shutil
 import warnings
-import hashlib
-import csv
-import glob
+from subprocess import Popen, PIPE
 
-from wheel.util import urlsafe_b64encode, open_for_csv, native
-
-from .libsana import tree_libs, stripped_lib_dict
+from .pycompat import string_types
+from .libsana import tree_libs, stripped_lib_dict, get_rp_stripper
 from .tools import (set_install_name, zip2dir, dir2zip,
-                    find_package_dirs, set_install_id)
+                    find_package_dirs, set_install_id, get_archs)
 from .tmpdirs import InTemporaryDirectory, InGivenDirectory
+from .wheeltools import rewrite_record, InWheel
 
 # Prefix for install_name_id of copied libraries
 DLC_PREFIX = '/DLC/'
@@ -278,8 +276,10 @@ def delocate_path(tree_path, lib_path,
         dict containing the (key, value) pairs of (``copied_lib_path``,
         ``dependings_dict``), where ``copied_lib_path`` is a library real path
         that was copied into `lib_sdir` of the wheel packages, and
-        ``dependings_dict`` is a dictionary giving the files in the wheel
-        depending on ``copied_lib_path``.
+        ``dependings_dict`` is a dictionary with key, value pairs where the key
+        is a file in the path depending on ``copied_lib_path``, and the value
+        is the ``install_name`` of ``copied_lib_path`` in the depending
+        library.
     """
     if not exists(lib_path):
         os.makedirs(lib_path)
@@ -291,55 +291,6 @@ def delocate_path(tree_path, lib_path,
     return copy_recurse(lib_path, copy_filt_func, copied)
 
 
-def rewrite_record(bdist_dir):
-    """ Rewrite RECORD file with hashes for all files in `wheel_sdir`
-
-    Copied from :method:`wheel.bdist_wheel.bdist_wheel.write_record`
-
-    Will also unsign wheel
-
-    Parameters
-    ----------
-    bdist_dir : str
-        Path of unpacked wheel file
-    """
-    info_dirs = glob.glob(pjoin(bdist_dir, '*.dist-info'))
-    if len(info_dirs) != 1:
-        raise DelocationError("Should be exactly one `*.dist_info` directory")
-    record_path = os.path.join(info_dirs[0], 'RECORD')
-    record_relpath = os.path.relpath(record_path, bdist_dir)
-    # Unsign wheel - because we're invalidating the record hash
-    sig_path = os.path.join(info_dirs[0], 'RECORD.jws')
-    if exists(sig_path):
-        os.unlink(sig_path)
-
-    def walk():
-        for dir, dirs, files in os.walk(bdist_dir):
-            for f in files:
-                yield os.path.join(dir, f)
-
-    def skip(path):
-        """Wheel hashes every possible file."""
-        return (path == record_relpath)
-
-    with open_for_csv(record_path, 'w+') as record_file:
-        writer = csv.writer(record_file)
-        for path in walk():
-            relpath = os.path.relpath(path, bdist_dir)
-            if skip(relpath):
-                hash = ''
-                size = ''
-            else:
-                with open(path, 'rb') as f:
-                    data = f.read()
-                digest = hashlib.sha256(data).digest()
-                hash = 'sha256=' + native(urlsafe_b64encode(digest))
-                size = len(data)
-            record_path = os.path.relpath(
-                path, bdist_dir).replace(os.path.sep, '/')
-            writer.writerow((record_path, hash, size))
-
-
 def _merge_lib_dict(d1, d2):
     """ Merges lib_dict `d2` into lib_dict `d1`
     """
@@ -355,13 +306,17 @@ def delocate_wheel(in_wheel,
                    out_wheel = None,
                    lib_sdir = '.dylibs',
                    lib_filt_func = _dylibs_only,
-                   copy_filt_func = filter_system_libs):
+                   copy_filt_func = filter_system_libs,
+                   require_archs = None,
+                   check_verbose = False,
+                  ):
     """ Update wheel by copying required libraries to `lib_sdir` in wheel
 
     Create `lib_sdir` in wheel tree only if we are copying one or more
     libraries.
 
-    Overwrite the wheel `wheel_fname` in-place.
+    If `out_wheel` is None (the default), overwrite the wheel `in_wheel`
+    in-place.
 
     Parameters
     ----------
@@ -383,6 +338,16 @@ def delocate_wheel(in_wheel,
         Default is callable rejecting only libraries beginning with
         ``/usr/lib`` or ``/System``.  None means copy all libraries. This will
         usually end up copying large parts of the system run-time.
+    require_archs : None or str or sequence, optional
+        If None, do no checks of architectures in libraries.  If sequence,
+        sequence of architectures (output from ``lipo -info``) that every
+        library in the wheels should have (e.g. ``['x86_64, 'i386']``). An
+        empty sequence results in checks that depended libraries have the same
+        archs as depending libraries.  If string, either "intel" (corresponds
+        to sequence ``['x86_64, 'i386']``) or name of required architecture
+        (e.g "i386" or "x86_64").
+    check_verbose : bool, optional
+        If True, print warning messages about missing required architectures
 
     Returns
     -------
@@ -390,9 +355,10 @@ def delocate_wheel(in_wheel,
         dict containing the (key, value) pairs of (``copied_lib_path``,
         ``dependings_dict``), where ``copied_lib_path`` is a library real path
         that was copied into `lib_sdir` of the wheel packages, and
-        ``dependings_dict`` is a dictionary giving the files in the wheel
-        depending on ``copied_lib_path``.  The files are relative to the wheel
-        root path.
+        ``dependings_dict`` is a dictionary with key, value pairs where the key
+        is a path in the wheel depending on ``copied_lib_path``, and the value
+        is the ``install_name`` of ``copied_lib_path`` in the depending
+        library. The filenames in the keys are relative to the wheel root path.
     """
     in_wheel = abspath(in_wheel)
     if out_wheel is None:
@@ -414,6 +380,15 @@ def delocate_wheel(in_wheel,
                         '{0} already exists in wheel'.format(lib_path))
                 if len(os.listdir(lib_path)) == 0:
                     shutil.rmtree(lib_path)
+                # Check architectures
+                if not require_archs is None:
+                    stop_fast = not check_verbose
+                    bads = check_archs(copied_libs, require_archs, stop_fast)
+                    if len(bads) != 0:
+                        if check_verbose:
+                            print(bads_report(bads, pjoin(tmpdir, 'wheel')))
+                        raise DelocationError(
+                            "Some missing architectures in wheel")
                 # Change install ids to be unique within Python space
                 install_id_root = DLC_PREFIX + package_path + '/'
                 for lib in copied_libs:
@@ -427,3 +402,148 @@ def delocate_wheel(in_wheel,
             dir2zip('wheel', out_wheel)
     wheel_dir = realpath(pjoin(tmpdir, 'wheel'))
     return stripped_lib_dict(all_copied, wheel_dir + os.path.sep)
+
+
+def patch_wheel(in_wheel, patch_fname, out_wheel=None):
+    """ Apply ``-p1`` style patch in `patch_fname` to contents of `in_wheel`
+
+    If `out_wheel` is None (the default), overwrite the wheel `in_wheel`
+    in-place.
+
+    Parameters
+    ----------
+    in_wheel : str
+        Filename of wheel to process
+    patch_fname : str
+        Filename of patch file.  Will be applied with ``patch -p1 <
+        patch_fname``
+    out_wheel : None or str
+        Filename of patched wheel to write.  If None, overwrite `in_wheel`
+    """
+    in_wheel = abspath(in_wheel)
+    patch_fname = abspath(patch_fname)
+    if out_wheel is None:
+        out_wheel = in_wheel
+    else:
+        out_wheel = abspath(out_wheel)
+    if not exists(patch_fname):
+        raise ValueError("patch file {0} does not exist".format(patch_fname))
+    with InWheel(in_wheel, out_wheel):
+        with open(patch_fname, 'rb') as fobj:
+            patch_proc = Popen(['patch', '-p1'],
+                               stdin = fobj,
+                               stdout = PIPE,
+                               stderr = PIPE)
+            stdout, stderr = patch_proc.communicate()
+            if patch_proc.returncode != 0:
+                raise RuntimeError("Patch failed with stdout:\n" +
+                                   stdout.decode('latin1'))
+
+
+def check_archs(copied_libs, require_archs=(), stop_fast=False):
+    """ Check compatibility of archs in `copied_libs` dict
+
+    Parameters
+    ----------
+    copied_libs : dict
+        dict containing the (key, value) pairs of (``copied_lib_path``,
+        ``dependings_dict``), where ``copied_lib_path`` is a library real path
+        that has been copied during delocation, and ``dependings_dict`` is a
+        dictionary with key, value pairs where the key is a path in the target
+        being delocated (a wheel or path) depending on ``copied_lib_path``, and
+        the value is the ``install_name`` of ``copied_lib_path`` in the
+        depending library.
+    require_archs : str or sequence, optional
+        Architectures we require to be present in all library files in wheel.
+        If an empty sequence, just check that depended libraries do have the
+        architectures of the depending libraries, with no constraints on what
+        these architectures are. If a sequence, then a set of required
+        architectures e.g. ``['i386', 'x86_64']`` to specify dual Intel
+        architectures.  If a string, then a standard architecture name as
+        returned by ``lipo -info`` or the string "intel", corresponding to the
+        sequence ``['i386', 'x86_64']``
+    stop_fast : bool, optional
+        Whether to give up collecting errors after the first
+
+    Returns
+    -------
+    bads : set
+        set of length 2 or 3 tuples. A length 2 tuple is of form
+        ``(depending_lib, missing_archs)`` meaning that an arch in
+        `require_archs` was missing from ``depending_lib``.  A length 3 tuple
+        is of form ``(depended_lib, depending_lib, missing_archs)`` where
+        ``depended_lib`` is the filename of the library depended on,
+        ``depending_lib`` is the library depending on ``depending_lib`` and
+        ``missing_archs`` is a set of missing architecture strings giving
+        architectures present in ``depending_lib`` and missing in
+        ``depended_lib``.  An empty set means all architectures were present as
+        required.
+    """
+    if isinstance(require_archs, string_types):
+        require_archs = (['i386', 'x86_64'] if require_archs == 'intel'
+                         else [require_archs])
+    require_archs = frozenset(require_archs)
+    bads = []
+    for depended_lib, dep_dict in copied_libs.items():
+        depended_archs = get_archs(depended_lib)
+        for depending_lib, install_name in dep_dict.items():
+            depending_archs = get_archs(depending_lib)
+            all_required = depending_archs | require_archs
+            all_missing = all_required.difference(depended_archs)
+            if len(all_missing) == 0:
+                continue
+            required_missing = require_archs.difference(depended_archs)
+            if len(required_missing):
+                bads.append((depending_lib, required_missing))
+            else:
+                bads.append((depended_lib, depending_lib, all_missing))
+            if stop_fast:
+                return set(bads)
+    return set(bads)
+
+
+def bads_report(bads, path_prefix=None):
+    """ Return a nice report of bad architectures in `bads`
+
+    Parameters
+    ----------
+    bads : set
+        set of length 2 or 3 tuples. A length 2 tuple is of form
+        ``(depending_lib, missing_archs)`` meaning that an arch in
+        `require_archs` was missing from ``depending_lib``.  A length 3 tuple
+        is of form ``(depended_lib, depending_lib, missing_archs)`` where
+        ``depended_lib`` is the filename of the library depended on,
+        ``depending_lib`` is the library depending on ``depending_lib`` and
+        ``missing_archs`` is a set of missing architecture strings giving
+        architectures present in ``depending_lib`` and missing in
+        ``depended_lib``.  An empty set means all architectures were present as
+        required.
+    path_prefix : None or str, optional
+        Path prefix to strip from ``depended_lib`` and ``depending_lib``. None
+        means do not strip anything.
+
+    Returns
+    -------
+    report : str
+        A nice report for printing
+    """
+    path_processor = ((lambda x : x) if path_prefix is None
+                      else get_rp_stripper(path_prefix))
+    reports = []
+    for result in bads:
+        if len(result) == 3:
+            depended_lib, depending_lib, missing_archs = result
+            reports.append("{0} needs {1} {2} missing from {3}".format(
+                path_processor(depending_lib),
+                'archs' if len(missing_archs) > 1 else 'arch',
+                ', '.join(sorted(missing_archs)),
+                path_processor(depended_lib)))
+        elif len(result) == 2:
+            depending_lib, missing_archs = result
+            reports.append("Required {0} {1} missing from {2}".format(
+                'archs' if len(missing_archs) > 1 else 'arch',
+                ', '.join(sorted(missing_archs)),
+                path_processor(depending_lib)))
+        else:
+            raise ValueError('Report tuple should be length 2 or 3')
+    return '\n'.join(sorted(reports))
